C----------------------------------------------------------------------
C     verify derivatives
C----------------------------------------------------------------------
C
      subroutine check_DFDU(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP)
C     ---------- ----------
			double precision U, PAR, F, DFDU, DFDP
			integer NDIM, ICP, IJAC
			dimension U(NDIM), PAR(*), F(NDIM), ICP(*)
      DIMENSION DFDU(NDIM,NDIM),DFDP(NDIM,*)
			double precision FL, FR, DF, u0, h
			dimension FL(NDIM), FR(NDIM)
			integer i, j

			do j=1,NDIM
				u0   = U(j)
				h    = 1.0D-8*(1.0D0+dabs(u0))
				U(j) = u0+h
				call FUNC(NDIM,U,ICP,PAR,0,FR,DFDU,DFDP)
				U(j) = u0-h
				call FUNC(NDIM,U,ICP,PAR,0,FL,DFDU,DFDP)
				do i=1,NDIM
					DF  = 0.5D0*(FR(i)-FL(i))/h
					DF  = dabs(DF-DFDU(i,j))/(1.0D0+dabs(F(i)))
					if(DF.GT.(10.0D0*h)) then
						print*,'DF/DU wrong at DFDU(',i,',',j,')?',
     #             ' relative difference=',DF
					endif
				enddo
				U(j) = u0
			enddo
C
			RETURN
			END
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
      subroutine check_DFDP(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP,NPAR)
C     ---------- ----------
			double precision U, PAR, F, DFDU, DFDP
			integer NDIM, ICP, IJAC, NPAR
			dimension U(NDIM), PAR(*), F(NDIM), ICP(*)
      DIMENSION DFDU(NDIM,NDIM),DFDP(NDIM,*)
			double precision FL, FR, DF, p0, h
			dimension FL(NDIM), FR(NDIM)
			integer i, j

			do j=1,NPAR
				p0     = PAR(j)
				h      = 1.0D-8*(1.0D0+dabs(p0))
				PAR(j) = p0+h
				call FUNC(NDIM,U,ICP,PAR,0,FR,DFDU,DFDP)
				PAR(j) = p0-h
				call FUNC(NDIM,U,ICP,PAR,0,FL,DFDU,DFDP)
				do i=1,NDIM
					DF  = 0.5D0*(FR(i)-FL(i))/h
					DF  = dabs(DF-DFDP(i,j))/(1.0D0+dabs(F(i)))
					if(DF.GT.(10.0D0*h)) then
						print*,'DF/DPAR wrong at DFDP(',i,',',j,')?',
     #             ' relative difference=',DF
					endif
				enddo
				PAR(j) = p0
			enddo
			
			RETURN
			END
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
      subroutine check_DBCDU(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC)
C     ---------- -----------
			double precision PAR, U0, U1, FB, DBC
			integer NDIM, ICP, NBC, IJAC
			dimension PAR(*), U0(NDIM), U1(NDIM), FB(NBC), ICP(*)
      DIMENSION DBC(NBC,*)
			double precision FBL, FBR, DFB, uu, h
			dimension FBL(NBC), FBR(NBC)
			integer i, j

			do j=1,NDIM
				uu    = U0(j)
				h     = 1.0D-8*(1.0D0+dabs(uu))
				U0(j) = uu+h
				call BCND(NDIM,PAR,ICP,NBC,U0,U1,FBR,0,DBC)
				U0(j) = uu-h
				call BCND(NDIM,PAR,ICP,NBC,U0,U1,FBL,0,DBC)
				do i=1,NBC
					DFB = 0.5D0*(FBR(i)-FBL(i))/h
					DFB = dabs(DFB-DBC(i,j))/(1.0D0+dabs(FB(i)))
					if(DFB.GT.(10.0D0*h)) then
						print*,'DBC/DU0 wrong at DBC(',i,',',j,')?',
     #             ' relative difference=',DFB
					endif
				enddo
				U0(j) = uu

				uu    = U1(j)
				h     = 1.0D-8*(1.0D0+dabs(uu))
				U1(j) = uu+h
				call BCND(NDIM,PAR,ICP,NBC,U0,U1,FBR,0,DBC)
				U1(j) = uu-h
				call BCND(NDIM,PAR,ICP,NBC,U0,U1,FBL,0,DBC)
				do i=1,NBC
					DFB = 0.5D0*(FBR(i)-FBL(i))/h
					DFB = dabs(DFB-DBC(i,NDIM+j))/(1.0D0+dabs(FB(i)))
					if(DFB.GT.(10.0D0*h)) then
						print*,'DBC/DU1 wrong at DBC(',i,',',NDIM+j,')?',
     #             ' relative difference=',DFB
					endif
				enddo
				U1(j) = uu
			enddo
			
			RETURN
			END
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
      subroutine check_DBCDP(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC,NPAR)
C     ---------- -----------
			double precision PAR, U0, U1, FB, DBC
			integer NDIM, ICP, NBC, IJAC, NPAR
			dimension PAR(*), U0(NDIM), U1(NDIM), FB(NBC), ICP(*)
      DIMENSION DBC(NBC,*)
			double precision FBL, FBR, DFB, p0, h
			dimension FBL(NBC), FBR(NBC)
			integer i, j

			do j=1,NPAR
				p0     = PAR(j)
				h      = 1.0D-8*(1.0D0+dabs(p0))
				PAR(j) = p0+h
				call BCND(NDIM,PAR,ICP,NBC,U0,U1,FBR,0,DBC)
				PAR(j) = p0-h
				call BCND(NDIM,PAR,ICP,NBC,U0,U1,FBL,0,DBC)
				do i=1,NBC
					DFB = 0.5D0*(FBR(i)-FBL(i))/h
					DFB = dabs(DFB-DBC(i,2*NDIM+j))/(1.0D0+dabs(FB(i)))
					if(DFB.GT.(10.0D0*h)) then
						print*,'DBC/DPAR wrong at DBC(',i,',',2*NDIM+j,')?',
     #             ' relative difference=',DFB
					endif
				enddo
				PAR(j) = p0
			enddo
			
			RETURN
			END
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
      subroutine check_DINTDU(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,
     #                        FI,IJAC,DINT)
C     ---------- ------------
			double precision PAR, U, UOLD, UDOT, UPOLD, FI, DINT
			integer NDIM, ICP, NINT, IJAC
			dimension PAR(*), U(NDIM), UOLD(NDIM), UDOT(NDIM), UPOLD(NDIM)
			dimension FI(NINT), DINT(NINT,*), ICP(*)
			double precision FIL, FIR, DFI, u0, h
			dimension FIL(NINT), FIR(NINT)
			integer i, j

			do j=1,NDIM
				u0   = U(j)
				h    = 1.0D-8*(1.0D0+dabs(u0))
				U(j) = u0+h
				call ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FIR,0,DINT)
				U(j) = u0-h
				call ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FIL,0,DINT)
				do i=1,NINT
					DFI = 0.5D0*(FIR(i)-FIL(i))/h
					DFI = dabs(DFI-DINT(i,j))/(1.0D0+dabs(FI(i)))
					if(DFI.GT.(10.0D0*h)) then
						print*,'DINT/DU wrong at DINT(',i,',',j,')?',
     #             ' relative difference=',DFI
					endif
				enddo
				U(j) = u0
			enddo
			
			RETURN
			END
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
      subroutine check_DINTDP(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,
     #                        FI,IJAC,DINT,NPAR)
C     ---------- ------------
			double precision PAR, U, UOLD, UDOT, UPOLD, FI, DINT
			integer NDIM, ICP, NINT, IJAC
			dimension PAR(*), U(NDIM), UOLD(NDIM), UDOT(NDIM), UPOLD(NDIM)
			dimension FI(NINT), DINT(NINT,*), ICP(*)
			double precision FIL, FIR, DFI, p0, h
			dimension FIL(NINT), FIR(NINT)
			integer i, j

			do j=1,NPAR
				p0     = PAR(j)
				h      = 1.0D-8*(1.0D0+dabs(p0))
				PAR(j) = p0+h
				call ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FIR,0,DINT)
				PAR(j) = p0-h
				call ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,UPOLD,FIL,0,DINT)
				do i=1,NINT
					DFI = 0.5D0*(FIR(i)-FIL(i))/h
					DFI = dabs(DFI-DINT(i,NDIM+j))/(1.0D0+dabs(FI(i)))
					if(DFI.GT.(10.0D0*h)) then
						print*,'DINT/DPAR wrong at DINT(',i,',',NDIM+j,')?',
     #             ' relative difference=',DFI
					endif
				enddo
				PAR(j) = p0
			enddo
			
			RETURN
			END
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
      subroutine check_DFSDU(NDIM,U,ICP,PAR,IJAC,FS,DFDU,DFDP)
C     ---------- -----------
			double precision U, PAR, FS, DFDU, DFDP
			integer NDIM, ICP, IJAC
			dimension U(NDIM), PAR(*), ICP(*)
      DIMENSION DFDU(NDIM),DFDP(*)
			double precision FSL, FSR, DFS, u0, h
			integer j

			do j=1,NDIM
				u0   = U(j)
				h    = 1.0D-8*(1.0D0+dabs(u0))
				U(j) = u0+h
				call FOPT(NDIM,U,ICP,PAR,0,FSR,DFDU,DFDP)
				U(j) = u0-h
				call FOPT(NDIM,U,ICP,PAR,0,FSL,DFDU,DFDP)
				DFS = 0.5D0*(FSR-FSL)/h
				DFS = dabs(DFS-DFDU(j))/(1.0D0+dabs(FS))
				if(DFS.GT.(10.0D0*h)) then
					print*,'DFS/DU wrong at DFDU(',j,')?',
     #             ' relative difference=',DFS
				endif
				U(j) = u0
			enddo
			
			RETURN
			END
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
      subroutine check_DFSDP(NDIM,U,ICP,PAR,IJAC,FS,DFDU,DFDP,NPAR)
C     ---------- -----------
			double precision U, PAR, FS, DFDU, DFDP
			integer NDIM, ICP, IJAC
			dimension U(NDIM), PAR(*), ICP(*)
      DIMENSION DFDU(NDIM),DFDP(*)
			double precision FSL, FSR, DFS, p0, h
			integer j

			do j=1,NPAR
				p0     = PAR(j)
				h      = 1.0D-8*(1.0D0+dabs(p0))
				PAR(j) = p0+h
				call FOPT(NDIM,U,ICP,PAR,0,FSR,DFDU,DFDP)
				PAR(j) = p0-h
				call FOPT(NDIM,U,ICP,PAR,0,FSL,DFDU,DFDP)
				DFS = 0.5D0*(FSR-FSL)/h
				DFS = dabs(DFS-DFDP(j))/(1.0D0+dabs(FS))
				if(DFS.GT.(10.0D0*h)) then
					print*,'DFS/DPAR wrong at DFDP(',j,')?',
     #             ' relative difference=',DFS
				endif
				PAR(j) = p0
			enddo
			
			RETURN
			END
C----------------------------------------------------------------------
C----------------------------------------------------------------------
