#ifndef _parser_h
#define _parser_h

void read_constants_file(const char * fname);
void write_constants_file(std::string IRS);
void read_rauto_args(int arg, int argn, const char *argv[]);
std::list<std::string> IRS_list();
int print_usage(std::string argname = "");

class parse_error {
	private:
		std::string msg;
		
	public:
		parse_error() {}
		parse_error(std::string m) : msg(m) {}
		parse_error(const char *m) : msg(m) {}
		virtual ~parse_error() {};
		virtual void print(const char * pname);
};

class arg_error : public parse_error {
	private:
		unsigned int Arg;
		std::string  OName;
	
	public:
		arg_error(int a, std::string n, std::string m="")
			: parse_error(m), Arg(a), OName(n) {}
		void print(const char * pname);
};

class too_many_args_error : public parse_error {
	private:
		unsigned int Arg;
		std::string OName;
	
	public:
		too_many_args_error(int a, std::string n) : Arg(a), OName(n) {}
		void print(const char * pname);
};

class invalid_arg : public parse_error {
	private:
		std::string token;
	
	public:
		invalid_arg(std::string t) : token(t) {}
		void print(const char * pname);
};

class cstdlib_error : public parse_error {
	private:
		int errnum;
		std::string msg;
	
	public:
		cstdlib_error(int e, const char *m) : errnum(e), msg(m) {}
		void print(const char * pname);
};

class stl_error : public parse_error {
	private:
		int row, col;
		std::string fname, msg, entry;
	
	public:
		stl_error(int r, int c, const char *n, const char *m, const char *e="")
			: row(r), col(c), fname(n), msg(m), entry(e) {}
		void print(const char * pname);
};

#endif
